package Encode::Newlines;
our $VERSION = '0.01';
our $AllowMixed = 0;

use strict;
use base qw(Encode::Encoding);
use constant CR => "\015";
use constant LF => "\012";
use constant CRLF => "\015\012";
use constant Native => (
    ($^O =~ /^(?:MSWin|cygwin|dos|os2)/) ? CRLF :
    ($^O =~ /^MacOS/) ? CR : LF
);

foreach my $name (qw( CR LF CRLF Native )) {
    no strict 'refs';
    $Encode::Encoding{$name} = bless {
        EOL => &$name,
        Name => $name,
    } => __PACKAGE__;
};

sub encode($$;$) {
    my ($obj, $str, $chk) = @_;

    if ($AllowMixed) {
        $str =~ s/(?:\015\012?|\012)/$obj->{EOL}/g;
    }
    elsif ($str =~ /((?:\015\012?|\012))/) {
        my $eol = $1;

        if ($eol eq CRLF) {
            require Carp;
            Carp::croak 'Mixed newlines'
                if $str =~ /\015(?!\012)|(?<!\015)\012/;
        }
        else {
            require Carp;
            Carp::croak 'Mixed newlines'
                if index($str, (($eol eq CR) ? LF : CR)) >= 0;
        }

        $str =~ s/$eol/$obj->{EOL}/g;
    }

    $_[1] = '' if $chk;
    return $str;
}

no warnings 'once';
*decode = \*encode;

1;
__END__

=head1 NAME

Encode::Newlines - Normalize line ending sequences

=head1 VERSION

This document describes version 0.01 of Encode::Newlines, released 
October 6, 2004.

=head1 SYNOPSIS

    use Encode;
    use Encode::Newlines;

    # Convert input and output streams to CRLF
    open IN, '<:raw:encoding(CRLF)', 'file.txt' or die $!;
    open OUT, '>:raw:encoding(CRLF)', 'file.txt' or die $!;

    # Convert to native newlines
    # Note that encode() and decode() are equivalent
    $native = encode(Native => $string);
    $native = decode(Native => $string);

    {
        # Allow mixed newlines in $mixed
        local $Encode::Newlines::AllowMixed = 1;
        $cr = encode(CR => $mixed);
    }

=head1 DESCRIPTION

This module provides four encodings: C<CR>, C<LF>, C<CRLF> and C<Native>,
to aid in normalizing line endings.  It is designed for the L<SVK> version
control system to support the C<svn:eol-style> property.

It converts whatever line endings the source uses to the designated newline
sequence, for both C<encode> and C<decode> operations.

If the source has an inconsistent line ending style, then a C<Mixed newlines>
exception is raised on behalf of the caller.  However, if the package variable
C<$Encode::Newlines::AllowMixed> is set to a true value, then it will silently
convert all three line endings.

=head1 TODO

An optional XS implemenation would be nice.

=head1 SEE ALSO

L<SVK>

=head1 AUTHORS

Autrijus Tang E<lt>autrijus@autrijus.orgE<gt>

=head1 COPYRIGHT

Copyright 2004 by Autrijus Tang E<lt>autrijus@autrijus.orgE<gt>.

This program is free software; you can redistribute it and/or 
modify it under the same terms as Perl itself.

See L<http://www.perl.com/perl/misc/Artistic.html>

=cut
